# from sprites.sprites import get_car, get_road
# get_road fxn takes direction parameter as strings
# i.e "NS" for north/south roads and "EW" for east/west
import os
from flow import settings
import pygame
import pyganim
import random

def get_fire():
    rects = [
        (0, 0, 75, 75),
        (75, 0, 75, 75),
        (160, 0, 75, 75),
        ]
    images = pyganim.getImagesFromSpriteSheet(
        os.path.join(settings.SPRITES_DIR, 'explosion_anim.png'),
        rects=rects)
    images = [pygame.transform.scale(image, (settings.CAR_SIZE, settings.CAR_SIZE))
              for image in images]
    frames = list(zip(images, [settings.EXPLOSION_RATE] * len(images)))
    fire = pyganim.PygAnimation(frames, loop=False)
    fire.play()
    return fire

def fill_space(x1, x2, y1, y2):
    width = x2 - x1
    height = y2 - y1

    spaces = []

    if width > 160 and height > 160:
        mix = pygame.Surface((width, height), pygame.SRCALPHA, 32)

        if width > 200 and height > 200:
            b = building(width//3, height//3)
            o = building(width//3, height//3, "office")
            g = get_grass(width//3, height//3)
            p = park(width//3, height//3)
            buildings = [b, o]
            greenspace = [g, p]
            mix.blit(random.choice(buildings), (0, 0))
            mix.blit(random.choice(buildings), (width//3, 0))
            mix.blit(random.choice(greenspace), (width*2//3, 0))
            mix.blit(random.choice(greenspace), (0, height//3))
            mix.blit(random.choice(buildings), (width//3, height//3))
            mix.blit(random.choice(buildings), (width*2//3, height//3))
            mix.blit(random.choice(buildings), (0, height*2//3))
            mix.blit(random.choice(greenspace), (width//3, height*2//3))
            mix.blit(random.choice(buildings), (width*2//3, height*2//3))
            spaces.append(mix)

        else:
            b = building(width//2, height//2)
            o = building(width//2, height//2, "office")
            g = get_grass(width//2, height//2)
            p = park(width//2, height//2)
            buildings = [b, o]
            greenspace = [g, p]
            mix.blit(random.choice(buildings), (0, 0))
            mix.blit(random.choice(greenspace), (width//2, 0))
            mix.blit(random.choice(greenspace), (0, height//2))
            mix.blit(random.choice(buildings), (width//2, height//2))
            spaces.append(mix)

    elif width > 160 and height <= 160:
        dual1 = pygame.Surface((width, height), pygame.SRCALPHA, 32)
        dual2 = pygame.Surface((width, height), pygame.SRCALPHA, 32)
        b = building(width//2, height)
        dual1.blit(b, (0, 0))
        dual2.blit(b, (width//2, 0))
        g = get_grass(width//2, height)
        dual1.blit(g, (width//2, 0))
        dual2.blit(g, (0, 0))
        spaces.append(dual1)
        spaces.append(dual2)
    elif height > 160 and width <= 160:
        dual1 = pygame.Surface((width, height), pygame.SRCALPHA, 32)
        dual2 = pygame.Surface((width, height), pygame.SRCALPHA, 32)
        b = building(width, height//2)
        dual2.blit(b, (0, height//2))
        dual1.blit(b, (0, 0))
        g = get_grass(width, height//2)
        dual1.blit(g, (0, height//2))
        dual2.blit(g, (0, 0))
        spaces.append(dual1)
        spaces.append(dual2)
    else:
        spaces.append(park(width, height))
        spaces.append(get_grass(width, height))
        spaces.append(building(width, height))
        spaces.append(building(width, height, "office"))
    return (random.choice(spaces))


def get_grass(width, height):
    tiles = [
        (0, 408, 16, 16),
    ]
    images = pyganim.getImagesFromSpriteSheet(
        os.path.join(settings.SPRITES_DIR, 'city.png'),
        rects=tiles)
    grass = pygame.Surface((width, height), pygame.SRCALPHA, 32)

    for i in range(0, height, 16):
        for j in range(0, width, 16):
            grass.blit(images[0], (j, i))
    return grass


def park(width, height):
    tiles = [
        # grass tiles 0 - 8
        (0, 425, 16, 16),
        (17, 425, 16, 16),
        (34, 425, 16, 16),
        (0, 442, 16, 16),
        (17, 442, 16, 16),
        (34, 442, 16, 16),
        (0, 459, 16, 16),
        (17, 459, 16, 16),
        (34, 459, 16, 16),
        # trees 9 - 12
        (526, 170, 16, 16),
        (526, 187, 16, 16),
        (543, 170, 16, 16),
        (543, 187, 16, 16),
    ]

    images = pyganim.getImagesFromSpriteSheet(
        os.path.join(settings.SPRITES_DIR, 'city.png'),
        rects=tiles)
    park = pygame.Surface((width, height), pygame.SRCALPHA, 32)

    # make top row
    park.blit(images[0], (0, 0))
    for i in range(16, (width), 16):
        park.blit(images[1], (i, 0))
    park.blit(images[2], ((width-16), 0))

    # make the middle green
    for i in range(16, height, 16):
        park.blit(images[3], (0, i))
        for j in range(16, width, 16):
            park.blit(images[4], (j, i))
            # more trees
            # if random.randint(1,21) % 5 == 0:
            #     park.blit(images[9], (j, (i-16)))
            #     park.blit(images[10], (j, i))
        park.blit(images[5], ((width-16), i))



    # make the bottom row
    park.blit(images[6], (0, (height-16)))
    for i in range(16, width, 16):
        park.blit(images[7], (i, (height-16)))
    park.blit(images[8], ((width-16), (height-16)))

    #trees
    park.blit(images[9], (0, (height-40)))
    park.blit(images[10], (0, (height-24)))
    for i in range(16 , height-16, 16):
        for j in range(16, width-16, 16):
            if random.randint(1,21) > 15:
                park.blit(images[9], (j, (i-16)))
                park.blit(images[10], (j, i))

    return park


def get_taillight():
    tiles = [
    (0, 0, 32, 16)
    ]
    images = pyganim.getImagesFromSpriteSheet(
        os.path.join(settings.SPRITES_DIR, 'brakes.png'),
        rects=tiles)
    light = pygame.Surface((16*2, 16), pygame.SRCALPHA, 32)
    light.blit(images[0], (0, 0))
    return light

def get_traffic_light(color=None):
    # creates a traffic light
    tiles = [
        (0, 0, 16, 16),  # green light
        # (8, 0, 16, 16),  # amber light
        (16, 0, 16, 16),  # red light
        # (24, 0, 16, 16),  # purple light
        # (48, 0, 16, 16),  # blank light
    ]

    images = pyganim.getImagesFromSpriteSheet(
        os.path.join(settings.SPRITES_DIR, 'glow_lights.png'),
        rects=tiles)
    light = pygame.Surface((20, 20), pygame.SRCALPHA, 32)

    if color == "green":
        light.blit(images[0], (0, 0))
    elif color == "red":
        light.blit(images[1], (0, 0))
    # elif color == "amber":
    #     light.blit(images[1], (0, 0))
    # elif color == "purple":
    #     light.blit(images[3], (0, 0))
    # else:
    #     light.blit(images[4], (0, 0))
    # light.blit(images[5], (0, 16))
    return light

def building(width, height, type=None):
    # creates a building
    if type=="office":
        tiles = [
        (221, 153, 16, 16), # 0 brick left corner
        (238, 153, 16, 16), # 1 brick bottom middle
        (255, 153, 16, 16), # 2 brick right corner
        (221, 153, 16, 16), # 3 brick left edge
        (238, 153, 16, 16), # 4 brick middle
        (255, 153, 16, 16), # 5 brick right edge

        (136, 0, 16, 16), # 6 roof top left
        (170, 0, 16, 16), # 7 roof top middle
        (153, 0, 16, 16), # 8 roof top right
        (136, 17, 16, 16), # 9 roof bottom left
        (170, 17, 16, 16), # 10 roof bottom middle
        (153, 17, 16, 16), # 11 roof bottom right

        (425, 408, 16, 16), # 12 door top left
        (442, 408, 16, 16), # 13 door top right
        (425, 425, 16, 16), # 14 door lower left
        (442, 425, 16, 16), # 15 door lower right
        ]
    else:
        tiles = [
            (17, 153, 16, 16), # 0 brick left corner
            (34, 153, 16, 16), # 1 brick bottom middle
            (51, 153, 16, 16), # 2 brick right corner
            (17, 85, 16, 16), # 3 brick left edge
            (34, 85, 16, 16), # 4 brick middle
            (51, 85, 16, 16), # 5 brick right edge
            (0, 0, 16, 16), # 6 roof top left
            (34, 0, 16, 16), # 7 roof top middle
            (17, 0, 16, 16), # 8 roof top right
            (0, 17, 16, 16), # 9 roof bottom left
            (34, 17, 16, 16), # 10 roof bottom middle
            (17, 17, 16, 16), # 11 roof bottom right

            (357, 459, 16, 16), # 12 door
            (441, 271, 16, 16), # 13 window

        ]

    images = pyganim.getImagesFromSpriteSheet(
        os.path.join(settings.SPRITES_DIR, 'city.png'),
        rects=tiles)
    building = pygame.Surface((width, height), pygame.SRCALPHA, 32)

    # make the middle bricks
    for i in range(32, height, 16):
        building.blit(images[3], (0, i))
        for j in range(16, width, 16):
            building.blit(images[4], (j, i))
        building.blit(images[5], ((width-16), i))

    # make the floor
    building.blit(images[0], (0, (height-16)))
    for i in range(16, width, 16):
        building.blit(images[1], (i, (height-16)))
    building.blit(images[2], ((width-16), (height-16)))

    # make a door
    if type == "office":
        building.blit(images[12], ((width/2-16), (height-32)))
        building.blit(images[13], ((width/2), (height-32)))
        building.blit(images[14], ((width/2-16), (height-16)))
        building.blit(images[15], ((width/2), (height-16)))
    else:
        building.blit(images[12], ((width/2-8), (height-16)))

    # make some windows
    if type != "office":
        for i in range(32, (height-32), 16):
            building.blit(images[13], ((width/5-8), i))
            building.blit(images[13], ((width*2/5-8), i))
            building.blit(images[13], ((width*3/5-8), i))
            building.blit(images[13], ((width*4/5-8), i))

    # make the roof
    building.blit(images[6], (0, 0))
    for i in range(16, (width), 16):
        building.blit(images[7], (i, 0))
    building.blit(images[8], ((width-16), 0))

    building.blit(images[9], (0, 16))
    for i in range(16, (width), 16):
        building.blit(images[10], (i, 16))
    building.blit(images[11], ((width-16), 16))

    return building


def get_car(color=None):
    # creates the car
    if color == "orange":
        tiles_car = [
            (561, 442, 16, 16),
            (578, 442, 16, 16),
            (561, 459, 16, 16),
            (578, 459, 16, 16)
        ]
    elif color == "green":
        tiles_car = [
            (561, 306, 16, 16),
            (578, 306, 16, 16),
            (561, 323, 16, 16),
            (578, 323, 16, 16)
        ]
    else:
        tiles_car = [
            (566, 376, 11, 14),
            (578, 376, 11, 14),
            (566, 391, 11, 15),
            (578, 391, 11, 15)
        ]

    images = pyganim.getImagesFromSpriteSheet(
        os.path.join(settings.SPRITES_DIR, 'city.png'),
        rects=tiles_car
    )
    car = pygame.Surface((22, 29), pygame.SRCALPHA, 32)
    for i, img in enumerate(images):
        width = i % 2 * 11
        y = i // 2 * 14
        car.blit(img, (width, y))
    return car


def get_NS_road(height):
    # create vertical road
    tiles_road = [
        (119, 340, 16, 16),  # side walk
        (187, 323, 16, 16),  # blank road
        (153, 357, 16, 16),  # top road with dash
        (170, 357, 16, 16),  # bottom road with dash
        (187, 323, 16, 16),  # blank road
        (119, 340, 16, 16),  # side walk

    ]
    images_road = pyganim.getImagesFromSpriteSheet(
        os.path.join(settings.SPRITES_DIR, 'city.png'),
        rects=tiles_road
    )
    road = pygame.Surface((16*len(tiles_road), height), pygame.SRCALPHA, 32)
    for i in range(0, height, 16):
        for j, img in enumerate(images_road):
            road.blit(img, (j*16, i))
    return road

def get_EW_road(height):
    #creates horizontal road
    tiles_road = [
             ( 170, 323, 16, 16),
             ( 170, 340, 16, 16),
             ( 187, 323, 16, 16),
             ( 136, 357, 16, 16)]
    images_road = pyganim.getImagesFromSpriteSheet(
        os.path.join(settings.SPRITES_DIR, 'city.png'), rects=tiles_road)
    road = pygame.Surface((height, 16*6), pygame.SRCALPHA, 32)
    for i in range(0, height, 16):
        road.blit(images_road[3], (i, 0))
        road.blit(images_road[2], (i, 16))
        road.blit(images_road[0], (i, 32))
        road.blit(images_road[1], (i, 48))
        road.blit(images_road[2], (i, 64))
        road.blit(images_road[3], (i, 80))
    return road


def get_road(height, direction):
    if direction in ["NS", "N", "S", "vertical"]:
        return get_NS_road(height)
    elif direction in ["EW", "E", "W", "horizontal"]:
        return get_EW_road(height)


def get_intersection():
    tiles = [
         (120, 340, 16, 16),  # sidewalk
         (187, 323, 16, 16),  # blank road
         (153, 357, 16, 16),  # road w right line
         (170, 357, 16, 16),  # road w left line
         (170, 323, 16, 16),  # road w top line
         (170, 340, 16, 16),  # road w bottom line
         (204, 323, 16, 16),  # middle part
         ]
    images = pyganim.getImagesFromSpriteSheet(
        os.path.join(settings.SPRITES_DIR, 'city.png'), rects=tiles)

    intersection = pygame.Surface((16*6, 16*6), pygame.SRCALPHA, 32)
    #row 1
    intersection.blit(images[0], (0, 0))
    intersection.blit(images[1], (16, 0))
    intersection.blit(images[2], (32, 0))
    intersection.blit(images[3], (48, 0))
    intersection.blit(images[1], (64, 0))
    intersection.blit(images[0], (80, 0))
    #row2
    intersection.blit(images[1], (0, 16))
    intersection.blit(images[1], (16, 16))
    intersection.blit(images[2], (32, 16))
    intersection.blit(images[3], (48, 16))
    intersection.blit(images[1], (64, 16))
    intersection.blit(images[1], (80, 16))
    #row3
    intersection.blit(images[4], (0, 32))
    intersection.blit(images[4], (16, 32))
    intersection.blit(images[1], (32, 32))
    intersection.blit(images[4], (48, 32))
    intersection.blit(images[4], (64, 32))
    intersection.blit(images[4], (80, 32))
    #row4
    intersection.blit(images[5], (0, 48))
    intersection.blit(images[5], (16, 48))
    intersection.blit(images[1], (32, 48))
    intersection.blit(images[5], (48, 48))
    intersection.blit(images[5], (64, 48))
    intersection.blit(images[5], (80, 48))
    #row5
    intersection.blit(images[1], (0, 64))
    intersection.blit(images[1], (16, 64))
    intersection.blit(images[2], (32, 64))
    intersection.blit(images[3], (48, 64))
    intersection.blit(images[1], (64, 64))
    intersection.blit(images[1], (80, 64))
    #row6
    intersection.blit(images[0], (0, 80))
    intersection.blit(images[1], (16, 80))
    intersection.blit(images[2], (32, 80))
    intersection.blit(images[3], (48, 80))
    intersection.blit(images[1], (64, 80))
    intersection.blit(images[0], (80, 80))

    intersection.blit(images[6], (40, 40))
    return intersection


def darken_and_gray(image):
    image = image.copy()
    for x in range(image.get_width()):
        for y in range(image.get_height()):
            pixel = image.get_at((x, y))
            new_value = sum(pixel[:3])//3
            new_value -= 50
            new_value = 0 if new_value < 0 else new_value
            image.set_at((x,y), (new_value, new_value, new_value, pixel[3]))
    return image
def colorize(image, newColor):
    """
    Create a "colorized" copy of a surface (replaces RGB values with the given color, preserving the per-pixel alphas of
    original).
    :param image: Surface to create a colorized copy of
    :param newColor: RGB color to use (original alpha values are preserved)
    :return: New colorized Surface instance
    """
    image = image.copy()

    # zero out RGB values
    # image.fill((0, 0, 0, 255), None, pygame.BLEND_RGBA_MULT)
    # add in new RGB values
    image.fill(newColor[0:3] + (0,), None, pygame.BLEND_RGBA_ADD)
    return image

    '''
        new_car = colorize(get_car(),(0, 191, 255))
        windowSurface.blit(new_car, (116, 100))
        new_car = colorize(get_car(),(191, 0, 255))
        windowSurface.blit(new_car, (116, 150))
        new_car = colorize(get_car(),(20, 20, 25))
        windowSurface.blit(new_car, (116, 200))
        new_car = colorize(get_car(),(20, 200, 25))
        windowSurface.blit(new_car, (116, 250))
    '''
