
class Difficulty:
    def __init__(self, name, value, delta=1):
        self.name = name
        self.value = value
        self.delta = delta

    def increase(self):
        self.value += self.delta
        return True

    def __repr__(self):
        return '<Difficulty Setting: {} = {}>'.format(self.name, self.value)


class PctDifficulty(Difficulty):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def increase(self):
        self.value = int(self.value * self.delta)
        return True


class MaxOutDifficulty(Difficulty):
    def __init__(self, *args, **kwargs):
        if 'max' in kwargs:
            self.max_diff = kwargs.pop('max')
        else:
            raise Exception('MaxOutDifficulty requires a max argument')
        super().__init__(*args, **kwargs)

    def increase(self):
        super().increase()
        if self.value > self.max_diff:
            self.value = self.max_diff
            return False
        else:
            return True


class MinOutDifficulty(Difficulty):
    def __init__(self, *args, **kwargs):
        if 'min' in kwargs:
            self.min_diff = kwargs.pop('min')
        else:
            raise Exception('MinOutDifficulty requires a min argument')
        super().__init__(*args, **kwargs)

    def increase(self):
        super().increase()
        if self.value < self.min_diff:
            self.value = self.min_diff
            return False
        else:
            return True
