from itertools import cycle
import pygame
from flow import settings, events
from flow.scenes import Intro, GameLevel, StatsPage
from flow.signal import Signal

from flow.assets import sound_loader


class SceneManager():
    def __init__(self):
        self.signals = Signal()
        self.scenes = [Intro()]
        self.current_scene = 0
        self.current = self.scenes[self.current_scene]
        self.finished = False

        self.level_loop = cycle([GameLevel, StatsPage])

    def update(self):
        for signal in self.signals.pop_signals():
            signal_type = signal[0]
            signal_args = list(signal[1:])
            if signal_type == "escape":
                print("Ok, bye")
                self.finished = True

            if signal_type == "next":
                if self.current_scene is None:
                    self.current_scene = 0
                else:
                    self.current_scene += 1

                self.current = self.get_next_scene(*signal_args)

            if signal_type == "add-car" and isinstance(self.current, GameLevel):
                self.current.add_car()

        self.current.update()

    def get_next_scene(self, *args):
        if self.current_scene < len(self.scenes):
            return self.scenes[self.current_scene](*args)
        else:
            return next(self.level_loop)(*args)

    def key_pressed(self, key):
        self.current.key_pressed(key)

    def mouse_pressed(self, pos):
        self.current.mouse_pressed(pos)

    def draw(self, screen):
        screen.fill(self.current.background_color)
        self.current.draw(screen)

    def fill(self, screen):
        self.current.fill(screen)

    def tick(self, clock):
        clock.tick(self.current.fps)


def play():
    pygame.init()
    pygame.font.init()

    pygame.mixer.init()
    pygame.mixer.set_num_channels(settings.MAX_SIMULTANEOUS_SOUNDS)
    pygame.mixer.set_reserved(1)  # Reserve one channel for background music

    screen = pygame.display.set_mode(settings.SCREEN_SIZE)
    pygame.display.set_caption("Destination: Irrelevant")
    sound_loader.play_background_music()

    scene_manager = SceneManager()
    clock = pygame.time.Clock()
    while not scene_manager.finished:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                Signal().send_signal("escape")
            if event.type == pygame.MOUSEBUTTONUP:
                scene_manager.mouse_pressed(pygame.mouse.get_pos())
            if event.type == pygame.KEYDOWN:
                scene_manager.key_pressed(event.key)
            if event.type == events.NEW_CAR_EVENT:
                Signal().send_signal("add-car")

        scene_manager.update()
        scene_manager.draw(screen)
        pygame.display.flip()
        scene_manager.tick(clock)

if __name__ == '__main__':
    play()
