import os
from flow import settings


class TerminalTextMixin:
    """
    Flashes text as if it's being typed
    """

    def __init__(self):
        if hasattr(self, 'text'):
            self._text = self.text
            self.line = 0
            self.char = 1
            self.finished = False
            self.flash_count = 12
        else:
            raise Exception('TerminalTextMixin Error: Child classes need a .text attribute')

    def flash(self, screen):
        if not self.finished:

            if self.flash_count % 5 == 0 and self.flash_count > 0:
                self.text = '$> ▐'
                self.flash_count -= 1
                self.write_text(screen)
                return
            elif self.flash_count % 5 > 0 and self.flash_count > 0:
                self.text = '$> '
                self.flash_count -= 1
                self.write_text(screen)
                return
            else:
                self.text = ''

            for i, line in enumerate(self._text):
                # We've already done this line
                if i < self.line:
                    self.text += '$> ' + line + '\n'
                # This is the line we are on
                elif i == self.line:
                    if self.char > len(line):
                        self.line += 1
                        self.char = 0
                    self.text += '$> ' + line[:self.char+1] + '\n'
                    self.char += 1
                    break

            if self.line == len(self._text) and self.char == 0:
                self.finished = True

        self.write_text(screen)
