import pygame
import random
from flow import settings, colors
from flow.assets.sprite_loader import get_road


class Road(pygame.sprite.Sprite):
    def __init__(self, position, orientation):
        super().__init__()
        self.orientation = orientation
        if orientation == 'vertical':
            self.y = 0
            self.x = position
            self.width = settings.ROAD_WIDTH
            self.length = settings.HEIGHT
        else:
            self.x = 0
            self.y = position
            self.width = settings.WIDTH
            self.length = settings.ROAD_WIDTH

        if self.orientation == 'horizontal':
            length = self.width
            size = (length, settings.ROAD_WIDTH)
        else:
            length = self.length
            size = (settings.ROAD_WIDTH, length)

        self.image = pygame.transform.scale(
                get_road(length, self.orientation),
                size
        )
        self.rect = self.image.get_rect()
        self.rect.x = self.x
        self.rect.y = self.y

    def entrypoints(self, size):
        offset = settings.ROAD_BUFFER
        if self.orientation == 'horizontal':
            entries = [
                (self.x, self.y + offset + size//2,  'E'),
                (self.width, self.y + settings.ROAD_WIDTH - offset - size//2, 'W')
            ]
        else:
            entries = [
                (self.x + settings.ROAD_WIDTH - offset - size//2, self.y, 'S'),
                (self.x + offset + size//2, self.length, 'N')
            ]

        return entries

    def __repr__(self):
        return f'{self.orientation} Road at {self.x}'

    def __str__(self):
        return self.__repr__()
