import pygame
from flow import colors, settings
from flow.assets.sprite_loader import get_traffic_light


class TrafficLight(pygame.sprite.Sprite):
    """
    """
    def __init__(self, x, y, stop_direction):
        super().__init__()

        self.stop_direction = stop_direction
        if self.stop_direction == 'N':
            self.x = x
            self.y = y + settings.ROAD_WIDTH

            self.width = settings.ROAD_WIDTH
            self.height = settings.CAR_SIZE + settings.CAR_SIZE // 2

        elif self.stop_direction == 'E':
            self.x = x - settings.CAR_SIZE - settings.CAR_SIZE // 2
            self.y = y

            self.width = settings.CAR_SIZE + settings.CAR_SIZE // 2
            self.height = settings.ROAD_WIDTH

        elif self.stop_direction == 'S':
            self.x = x
            self.y = y - settings.CAR_SIZE - settings.CAR_SIZE // 2

            self.width = settings.ROAD_WIDTH
            self.height = settings.CAR_SIZE + settings.CAR_SIZE // 2

        elif self.stop_direction == 'W':
            self.x = x + settings.ROAD_WIDTH
            self.y = y

            self.width = settings.CAR_SIZE + settings.CAR_SIZE // 2
            self.height = settings.ROAD_WIDTH

        self.pos = self.x, self.y
        self.dims = self.width, self.height

        self.rect = pygame.Rect(self.x, self.y, self.width, self.height)

        self.image = pygame.Surface(self.dims, pygame.SRCALPHA, 32)

        self.is_red = True
        self.update()

    def mouse_pressed(self):
        self.is_red = not self.is_red
        self.update()

    def update(self):
        if self.is_red:
            light = get_traffic_light("red")
        else:
            light = get_traffic_light("green")

        if self.stop_direction == 'N':
            pygame.draw.rect(
                self.image,
                colors.DARKGREY,
                (0, 0, self.width, self.height // 10))
            for i in range(3):
                self.image.blit(light, (2+i*20, 0))

        elif self.stop_direction == 'E':
            pygame.draw.rect(
                self.image,
                colors.DARKGREY,
                (self.width - self.width // 10, 0, self.width // 10, self.height)
            )
            light = pygame.transform.rotate(light, 270)
            for i in range(3):
                self.image.blit(light, (12, 2+i*20))

        elif self.stop_direction == 'S':
            pygame.draw.rect(
                self.image,
                colors.DARKGREY,
                (0, self.height - self.height // 10, self.width, self.height // 10)
            )
            light = pygame.transform.rotate(light, 180)
            for i in range(3):
                self.image.blit(light, (2+i*20, 12))

        elif self.stop_direction == 'W':
            pygame.draw.rect(
                self.image,
                colors.DARKGREY,
                (0, 0, self.width // 10, self.height)
            )
            light = pygame.transform.rotate(light, 90)
            for i in range(3):
                self.image.blit(light, (0, 2+i*20))
