import os
import random
import pygame
from flow import settings


def random_pos(offset=0):
    return (
        random.randint(offset, settings.WIDTH-offset),
        random.randint(offset, settings.HEIGHT-offset)
    )


def random_color(min=0, max=255):
    return (
        random.randint(0, max),
        random.randint(0, max),
        random.randint(0, max),
    )


IMAGE_LIBRARY = {}


def get_image(path):
    image = IMAGE_LIBRARY.get(path)
    if image is None:
        canonicalized_path = path.replace('/', os.sep).replace('\\', os.sep)
        image = pygame.image.load(canonicalized_path)
        IMAGE_LIBRARY[path] = image
    return image


DIALOGUE_LIBRARY = {}


def load_dialogue(name):
    path = '{}.txt'.format(name)
    dialogue = DIALOGUE_LIBRARY.get(name)
    if dialogue is None:
        safe_path = path.replace('/', os.sep).replace('\\', os.sep)
        with open(os.path.join(settings.DIALOGUE_DIR, safe_path), 'r') as f:
            dialogue = f.read().strip().split('\n')
        DIALOGUE_LIBRARY[name] = dialogue
    return dialogue


# draw some text into an area of a surface
# automatically wraps words
# returns any text that didn't get blitted
def drawText(surface, text, color, rect, font, aa=False, bkg=None):
    """
    from https://www.pygame.org/wiki/TextWrap
    """
    rect = pygame.Rect(rect)
    y = rect.top
    lineSpacing = 10

    # get the height of the font
    fontHeight = font.size("Tg")[1]

    while text:
        i = 1

        # determine if the row of text will be outside our area
        if y + fontHeight > rect.bottom:
            break

        # determine maximum width of line
        while font.size(text[:i])[0] < rect.width and i < len(text) and text[i] != '\n':
            i += 1

        # if we've wrapped the text, then adjust the wrap to the last word
        if i < len(text) and text[i] != '\n':
            i = text.rfind(" ", 0, i) + 1

        # render the  to the surface
        if bkg:
            image = font.render(text[:i], 1, color, bkg)
            image.set_colorkey(bkg)
        else:
            image = font.render(text[:i], aa, color)

        surface.blit(image, (rect.left, y))
        y += fontHeight + lineSpacing

        # remove the text we just blitted
        text = text[i:]

        if len(text) >= 1 and text[0] == '\n':
            text = text[1:]

    return text


def drawText2(surface, font, color, text, rect, aa=False, bkg=None):
    """
    from https://www.pygame.org/wiki/TextWrap
    """

    rect = pygame.Rect(rect)
    y = rect.top

    def blit(line, h):
        image = font.render(line, aa, color, bkg)
        surface.blit(image, (rect.left, h))

    # get the height of the font
    font_height = font.size("Tg")[1]
    line_break = None
    lines = text.split('\n')

    for line in lines:
        line_width = 0
        new_line = ''
        words = line.split(' ')
        flush = False

        for i, word in enumerate(words):
            line_width = font.size(new_line)[0]

            if i == len(words) - 1 and line_width + font.size(word)[0] <= rect.width:
                new_line += word
                line_break = True

            elif i == len(words) - 1 and line_width + font.size(word)[0] > rect.width:
                line_break = True
                flush = True

            elif i < len(words) - 1 and line_width + font.size(word + ' ')[0] <= rect.width:
                new_line += word + ' '
                line_break = False
            else:
                line_break = True

            if line_break:
                image = font.render(new_line, 1, color)
                surface.blit(image, (rect.left, y))
                y += font_height + settings.LINE_SPACING
                new_line = word + ' '

            if flush:
                blit(word, y)
                y += font_height + settings.LINE_SPACING
                new_line = ''
                flush = False
